@@ -1,21 +1,20 @@
 /* gpgme-w32spawn.c - Wrapper to spawn a process under Windows.
- * Copyright (C) 2008 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+   Copyright (C) 2008 g10 Code GmbH
+
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -29,23 +28,23 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <string.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdint.h>
 #include <process.h>
-
-#include "priv-io.h"
-
-
-/* Name of this program.  */
-#define PGM "gpgme-w32spawn"
+#include <windows.h>
 
 
 
+struct spawn_fd_item_s
+{
+  int handle;
+  int dup_to;
+  int peer_name;
+  int arg_loc;
+};
+
+
 static char *
 build_commandline (char **argv)
 {
@@ -53,7 +52,7 @@ build_commandline (char **argv)
   int n = 0;
   char *buf;
   char *p;
-
+  
   /* We have to quote some things because under Windows the program
      parses the commandline and does some unquoting.  We enclose the
      whole argument in double-quotes, and escape literal double-quotes
@@ -102,7 +101,7 @@ build_commandline (char **argv)
 
 
 int
-my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
+my_spawn (char **argv, struct spawn_fd_item_s *fd_list)
 {
   SECURITY_ATTRIBUTES sec_attr;
   PROCESS_INFORMATION pi =
@@ -122,18 +121,20 @@ my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
   int duped_stdout = 0;
   int duped_stderr = 0;
   HANDLE hnul = INVALID_HANDLE_VALUE;
+  /* FIXME.  */
+  int debug_me = 0;
 
   i = 0;
   while (argv[i])
     {
-      fprintf (stderr, PGM": argv[%2i] = %s\n", i, argv[i]);
+      fprintf (stderr, "argv[%2i] = %s\n", i, argv[i]);
       i++;
     }
 
   memset (&sec_attr, 0, sizeof sec_attr);
   sec_attr.nLength = sizeof sec_attr;
   sec_attr.bInheritHandle = FALSE;
-
+  
   arg_string = build_commandline (argv);
   if (!arg_string)
     return -1;
@@ -141,40 +142,40 @@ my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
   memset (&si, 0, sizeof si);
   si.cb = sizeof (si);
   si.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
-  si.wShowWindow = (flags & IOSPAWN_FLAG_SHOW_WINDOW) ? SW_SHOW : SW_HIDE;
+  si.wShowWindow = debug_me ? SW_SHOW : SW_HIDE;
   si.hStdInput = GetStdHandle (STD_INPUT_HANDLE);
   si.hStdOutput = GetStdHandle (STD_OUTPUT_HANDLE);
   si.hStdError = GetStdHandle (STD_ERROR_HANDLE);
 
-  fprintf (stderr, PGM": spawning: %s\n", arg_string);
+  fprintf (stderr, "spawning: %s\n", arg_string);
 
-  for (i = 0; fd_list[i].fd != -1; i++)
+  for (i = 0; fd_list[i].handle != -1; i++)
     {
       /* The handle already is inheritable.  */
       if (fd_list[i].dup_to == 0)
 	{
 	  si.hStdInput = (HANDLE) fd_list[i].peer_name;
 	  duped_stdin = 1;
-	  fprintf (stderr, PGM": dup 0x%x to stdin\n", fd_list[i].peer_name);
+	  fprintf (stderr, "dup 0x%x to stdin\n", fd_list[i].peer_name);
         }
       else if (fd_list[i].dup_to == 1)
 	{
 	  si.hStdOutput = (HANDLE) fd_list[i].peer_name;
 	  duped_stdout = 1;
-	  fprintf (stderr, PGM": dup 0x%x to stdout\n", fd_list[i].peer_name);
+	  fprintf (stderr, "dup 0x%x to stdout\n", fd_list[i].peer_name);
         }
       else if (fd_list[i].dup_to == 2)
 	{
 	  si.hStdError = (HANDLE) fd_list[i].peer_name;
 	  duped_stderr = 1;
-	  fprintf (stderr, PGM":dup 0x%x to stderr\n", fd_list[i].peer_name);
+	  fprintf (stderr, "dup 0x%x to stderr\n", fd_list[i].peer_name);
         }
     }
-
+  
   if (!duped_stdin || !duped_stdout || !duped_stderr)
     {
       SECURITY_ATTRIBUTES sa;
-
+      
       memset (&sa, 0, sizeof sa);
       sa.nLength = sizeof sa;
       sa.bInheritHandle = TRUE;
@@ -202,8 +203,9 @@ my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
       if (!duped_stderr)
 	si.hStdError = hnul;
     }
-
-  cr_flags |= CREATE_SUSPENDED;
+  
+  cr_flags |= CREATE_SUSPENDED; 
+  cr_flags |= DETACHED_PROCESS;
   if (!CreateProcessA (argv[0],
 		       arg_string,
 		       &sec_attr,     /* process security attributes */
@@ -226,37 +228,10 @@ my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
   /* Close the /dev/nul handle if used.  */
   if (hnul != INVALID_HANDLE_VALUE)
     CloseHandle (hnul);
-
-  for (i = 0; fd_list[i].fd != -1; i++)
-    CloseHandle ((HANDLE) fd_list[i].fd);
-
-  if (flags & IOSPAWN_FLAG_ALLOW_SET_FG)
-    {
-      static int initialized;
-      static BOOL (WINAPI * func)(DWORD);
-      void *handle;
-
-      if (!initialized)
-        {
-          /* Available since W2000; thus we dynload it.  */
-          initialized = 1;
-          handle = LoadLibrary ("user32.dll");
-          if (handle)
-            {
-              func = GetProcAddress (handle, "AllowSetForegroundWindow");
-              if (!func)
-                FreeLibrary (handle);
-            }
-        }
-
-      if (func)
-        {
-          int rc = func (pi.dwProcessId);
-          fprintf (stderr, PGM": AllowSetForegroundWindow(%d): rc=%d\n",
-                   (int)pi.dwProcessId, rc);
-        }
-    }
-
+  
+  for (i = 0; fd_list[i].handle != -1; i++)
+    CloseHandle ((HANDLE) fd_list[i].handle);
+  
   ResumeThread (pi.hThread);
   CloseHandle (pi.hThread);
   CloseHandle (pi.hProcess);
@@ -268,14 +243,13 @@ my_spawn (char **argv, struct spawn_fd_item_s *fd_list, unsigned int flags)
 #define MAX_TRANS 10
 
 int
-translate_get_from_file (const char *trans_file,
-			 struct spawn_fd_item_s *fd_list,
-                         unsigned int *r_flags)
+translate_get_from_file (const char *trans_file, 
+			 struct spawn_fd_item_s *fd_list)
 {
   /* Hold roughly MAX_TRANS triplets of 64 bit numbers in hex
      notation: "0xFEDCBA9876543210".  10*19*4 - 1 = 759.  This plans
      ahead for a time when a HANDLE is 64 bit.  */
-#define BUFFER_MAX 810
+#define BUFFER_MAX 800
 
   char line[BUFFER_MAX + 1];
   char *linep;
@@ -283,8 +257,6 @@ translate_get_from_file (const char *trans_file,
   int res;
   int fd;
 
-  *r_flags = 0;
-
   fd = open (trans_file, O_RDONLY);
   if (fd < 0)
     return -1;
@@ -297,12 +269,10 @@ translate_get_from_file (const char *trans_file,
 
   line[BUFFER_MAX] = '\0';
   linep = strchr (line, '\n');
-  if (linep)
-    {
-      if (linep > line && linep[-1] == '\r')
-        linep--;
-      *linep = '\0';
-    }
+  if (linep > line && linep[-1] == '\r')
+    linep--;
+  *linep = '\0';
+
   linep = line;
 
   /* Now start to read mapping pairs.  */
@@ -319,21 +289,6 @@ translate_get_from_file (const char *trans_file,
 	linep++;
       if (*linep == '\0')
 	break;
-      if (!idx && *linep == '~')
-        {
-          /* Spawn flags have been passed.  */
-          linep++;
-          *r_flags = strtoul (linep, &tail, 0);
-          if (tail == NULL || ! (*tail == '\0' || isspace (*tail)))
-            break;
-          linep = tail;
-
-          while (isspace (*((unsigned char *)linep)))
-            linep++;
-          if (*linep == '\0')
-            break;
-        }
-
       from = strtoul (linep, &tail, 0);
       if (tail == NULL || ! (*tail == '\0' || isspace (*tail)))
 	break;
@@ -366,12 +321,12 @@ translate_get_from_file (const char *trans_file,
 	break;
       linep = tail;
 
-      fd_list[idx].fd = from;
+      fd_list[idx].handle = from;
       fd_list[idx].dup_to = dup_to;
       fd_list[idx].peer_name = to;
       fd_list[idx].arg_loc = loc;
     }
-  fd_list[idx].fd = -1;
+  fd_list[idx].handle = -1;
   fd_list[idx].dup_to = -1;
   fd_list[idx].peer_name = -1;
   fd_list[idx].arg_loc = 0;
@@ -384,14 +339,13 @@ translate_get_from_file (const char *trans_file,
    FD_LIST (which must be MAX_TRANS+1 large).  */
 char **
 translate_handles (const char *trans_file, const char * const *argv,
-		   struct spawn_fd_item_s *fd_list, unsigned int *r_flags)
+		   struct spawn_fd_item_s *fd_list)
 {
   int res;
   int idx;
-  int n_args;
   char **args;
 
-  res = translate_get_from_file (trans_file, fd_list, r_flags);
+  res = translate_get_from_file (trans_file, fd_list);
   if (res < 0)
     return NULL;
 
@@ -405,9 +359,8 @@ translate_handles (const char *trans_file, const char * const *argv,
 	return NULL;
     }
   args[idx] = NULL;
-  n_args = idx;
 
-  for (idx = 0; fd_list[idx].fd != -1; idx++)
+  for (idx = 0; fd_list[idx].handle != -1; idx++)
     {
       char buf[25];
       int aidx;
@@ -416,12 +369,6 @@ translate_handles (const char *trans_file, const char * const *argv,
       if (aidx == 0)
 	continue;
 
-      if (aidx >= n_args)
-        {
-	  fprintf (stderr, PGM": translation file does not match args\n");
-          return NULL;
-        }
-
       args[aidx] = malloc (sizeof (buf));
       /* We currently disable translation for stdin/stdout/stderr.  We
 	 assume that the spawned program handles 0/1/2 specially
@@ -447,7 +394,6 @@ main (int argc, const char * const *argv)
   int rc = 0;
   char **argv_spawn;
   struct spawn_fd_item_s fd_list[MAX_TRANS + 1];
-  unsigned int flags;
 
   if (argc < 3)
     {
@@ -455,7 +401,7 @@ main (int argc, const char * const *argv)
       goto leave;
     }
 
-  argv_spawn = translate_handles (argv[1], &argv[2], fd_list, &flags);
+  argv_spawn = translate_handles (argv[1], &argv[2], fd_list);
   if (!argv_spawn)
     {
       rc = 2;
@@ -465,10 +411,10 @@ main (int argc, const char * const *argv)
   /* Using execv does not replace the existing program image, but
      spawns a new one and daemonizes it, confusing the command line
      interpreter.  So we have to use spawnv.  */
-  rc = my_spawn (argv_spawn, fd_list, flags);
+  rc = my_spawn (argv_spawn, fd_list);
   if (rc < 0)
     {
-      fprintf (stderr, PGM": executing `%s' failed: %s\n",
+      fprintf (stderr, "gpgwrap: executing `%s' failed: %s\n",
 	       argv[0], strerror (errno));
       rc = 2;
       goto leave;
@@ -476,12 +422,12 @@ main (int argc, const char * const *argv)
 
  leave:
   if (rc)
-    fprintf (stderr, PGM": internal error\n");
+    fprintf (stderr, "gpg-w32spawn: internal error\n");
   /* Always try to delete the temporary file.  */
   if (argc >= 2)
     {
       if (DeleteFile (argv[1]) == 0)
-	fprintf (stderr, PGM": failed to delete %s: ec=%ld\n",
+	fprintf (stderr, "Failed to delete %s: ec=%ld\n",
 		 argv[1], GetLastError ());
     }
   return rc;
