@@ -1,28 +1,27 @@
 /* version.c - Version check routines.
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 g10 Code GmbH
+ 
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
-#include <stdlib.h>
 #include <string.h>
 #include <limits.h>
 #include <ctype.h>
@@ -33,24 +32,14 @@
 #include "gpgme.h"
 #include "priv-io.h"
 #include "debug.h"
-#include "context.h"
 
-/* For _gpgme_sema_subsystem_init and _gpgme_status_init.  */
+/* For _gpgme_sema_subsystem_init ().  */
 #include "sema.h"
-#include "util.h"
 
 #ifdef HAVE_ASSUAN_H
 #include "assuan.h"
 #endif
 
-#ifdef HAVE_W32_SYSTEM
-#include "windows.h"
-#endif
-
-/* We implement this function, so we have to disable the overriding
-   macro.  */
-#undef gpgme_check_version
-
 
 /* Bootstrap the subsystems needed for concurrent operation.  This
    must be done once at startup.  We can not guarantee this using a
@@ -65,38 +54,26 @@ do_subsystem_inits (void)
   if (done)
     return;
 
-#ifdef HAVE_W32_SYSTEM
+  _gpgme_sema_subsystem_init ();
+#ifdef HAVE_ASSUAN_H
+  assuan_set_assuan_log_level (0);
+  assuan_set_assuan_err_source (GPG_ERR_SOURCE_GPGME);
+#endif /*HAVE_ASSUAN_H*/
+  _gpgme_debug_subsystem_init ();
+  _gpgme_io_subsystem_init ();
+#if defined(HAVE_W32_SYSTEM) && defined(HAVE_ASSUAN_H)
   /* We need to make sure that the sockets are initialized.  */
   {
     WSADATA wsadat;
-
+    
     WSAStartup (0x202, &wsadat);
   }
-#endif
-
-  _gpgme_debug_subsystem_init ();
-  _gpgme_io_subsystem_init ();
-  _gpgme_status_init ();
+#endif /*HAVE_W32_SYSTEM && HAVE_ASSUAN_H*/
 
   done = 1;
 }
 
 
-/* Put vesion information into the binary.  */
-static const char *
-cright_blurb (void)
-{
-  static const char blurb[] =
-    "\n\n"
-    "This is GPGME " PACKAGE_VERSION " - The GnuPG Made Easy library\n"
-    CRIGHTBLURB
-    "\n"
-    "(" BUILD_REVISION " " BUILD_TIMESTAMP ")\n"
-    "\n\n";
-  return blurb;
-}
-
-
 /* Read the next number in the version string STR and return it in
    *NUMBER.  Return a pointer to the tail of STR after parsing, or
    *NULL if the version string was invalid.  */
@@ -123,7 +100,7 @@ parse_version_number (const char *str, int *number)
 /* Parse the version string STR in the format MAJOR.MINOR.MICRO (for
    example, 9.3.2) and return the components in MAJOR, MINOR and MICRO
    as integers.  The function returns the tail of the string that
-   follows the version number.  This might be the empty string if there
+   follows the version number.  This might be te empty string if there
    is nothing following the version number, or a patchlevel.  The
    function returns NULL if the version string is not valid.  */
 static const char *
@@ -173,7 +150,7 @@ _gpgme_compare_versions (const char *my_version,
 
   if (my_major > rq_major
       || (my_major == rq_major && my_minor > rq_minor)
-      || (my_major == rq_major && my_minor == rq_minor
+      || (my_major == rq_major && my_minor == rq_minor 
 	  && my_micro > rq_micro)
       || (my_major == rq_major && my_minor == rq_minor
 	  && my_micro == rq_micro && strcmp (my_plvl, rq_plvl) >= 0))
@@ -195,53 +172,16 @@ _gpgme_compare_versions (const char *my_version,
 const char *
 gpgme_check_version (const char *req_version)
 {
-  const char *result;
   do_subsystem_inits ();
 
   /* Catch-22: We need to get at least the debug subsystem ready
-     before using the trace facility.  If we won't the trace would
+     before using the tarce facility.  If we won't the tarce would
      automagically initialize the debug system with out the locks
      being initialized and missing the assuan log level setting. */
-  TRACE (DEBUG_INIT, "gpgme_check_version", NULL,
-	  "req_version=%s, VERSION=%s",
-          req_version? req_version:"(null)", VERSION);
-
-  result = _gpgme_compare_versions (VERSION, req_version) ? VERSION : NULL;
-  if (result != NULL)
-    _gpgme_selftest = 0;
-
-  return result;
-}
-
-/* Check the version and also at runtime if the struct layout of the
-   library matches the one of the user.  This is particular useful for
-   Windows targets (-mms-bitfields).  */
-const char *
-gpgme_check_version_internal (const char *req_version,
-			      size_t offset_sig_validity)
-{
-  const char *result;
-
-  if (req_version && req_version[0] == 1 && req_version[1] == 1)
-    return cright_blurb ();
-  result = gpgme_check_version (req_version);
-  if (result == NULL)
-    return result;
-
-  /* Catch-22, see above.  */
-  TRACE (DEBUG_INIT, "gpgme_check_version_internal", NULL,
-	  "req_version=%s, offset_sig_validity=%zu",
-	  req_version ? req_version : "(null)", offset_sig_validity);
-
-  if (offset_sig_validity != offsetof (struct _gpgme_signature, validity))
-    {
-      TRACE (DEBUG_INIT, "gpgme_check_version_internal", NULL,
-	      "offset_sig_validity mismatch: expected %i",
-             (int)offsetof (struct _gpgme_signature, validity));
-      _gpgme_selftest = GPG_ERR_SELFTEST_FAILED;
-    }
-
-  return result;
+  TRACE2 (DEBUG_INIT, "gpgme_check_version: ", 0,
+	  "req_version=%s, VERSION=%s", req_version, VERSION);
+ 
+  return _gpgme_compare_versions (VERSION, req_version) ? VERSION : NULL;
 }
 
 
@@ -249,7 +189,7 @@ gpgme_check_version_internal (const char *req_version,
 
 /* Extract the version string of a program from STRING.  The version
    number is expected to be in GNU style format:
-
+   
      foo 1.2.3
      foo (bar system) 1.2.3
      foo 1.2.3 cruft
@@ -306,7 +246,7 @@ _gpgme_get_program_version (const char *const file_name)
   char *mark = NULL;
   int rp[2];
   int nread;
-  char *argv[] = {NULL /* file_name */, (char*)"--version", 0};
+  char *argv[] = {NULL /* file_name */, "--version", 0};
   struct spawn_fd_item_s cfd[] = { {-1, 1 /* STDOUT_FILENO */, -1, 0},
 				   {-1, -1} };
   int status;
@@ -320,8 +260,7 @@ _gpgme_get_program_version (const char *const file_name)
 
   cfd[0].fd = rp[1];
 
-  status = _gpgme_io_spawn (file_name, argv,
-                            IOSPAWN_FLAG_DETACHED, cfd, NULL, NULL, NULL);
+  status = _gpgme_io_spawn (file_name, argv, cfd, NULL);
   if (status < 0)
     {
       _gpgme_io_close (rp[0]);
@@ -338,7 +277,7 @@ _gpgme_get_program_version (const char *const file_name)
 	  mark = strchr (&line[linelen], '\n');
 	  if (mark)
 	    {
-	      if (mark > &line[0] && mark[-1] == '\r')
+	      if (mark > &line[0] && *mark == '\r')
 		mark--;
 	      *mark = '\0';
 	      break;
