@@ -1,23 +1,23 @@
-/* wait-global.c
- * Copyright (C) 2000 Werner Koch (dd9jn)
- * Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+/* wait-global.c 
+   Copyright (C) 2000 Werner Koch (dd9jn)
+   Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
+ 
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+   02111-1307, USA.  */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -34,7 +34,6 @@
 #include "wait.h"
 #include "priv-io.h"
 #include "ops.h"
-#include "debug.h"
 
 /* The global event loop is used for all asynchronous operations
    (except key listing) for which no user I/O callbacks are specified.
@@ -75,7 +74,6 @@ struct ctx_list_item
   gpgme_ctx_t ctx;
   /* The status is set when the ctx is moved to the done list.  */
   gpgme_error_t status;
-  gpgme_error_t op_err;
 };
 
 /* The active list contains all contexts that are in the global event
@@ -97,7 +95,7 @@ ctx_active (gpgme_ctx_t ctx)
 {
   struct ctx_list_item *li = malloc (sizeof (struct ctx_list_item));
   if (!li)
-    return gpg_error_from_syserror ();
+    return gpg_error_from_errno (errno);
   li->ctx = ctx;
 
   LOCK (ctx_list_lock);
@@ -114,7 +112,7 @@ ctx_active (gpgme_ctx_t ctx)
 
 /* Enter the context CTX into the done list with status STATUS.  */
 static void
-ctx_done (gpgme_ctx_t ctx, gpgme_error_t status, gpgme_error_t op_err)
+ctx_done (gpgme_ctx_t ctx, gpgme_error_t status)
 {
   struct ctx_list_item *li;
 
@@ -133,7 +131,6 @@ ctx_done (gpgme_ctx_t ctx, gpgme_error_t status, gpgme_error_t op_err)
     ctx_active_list = li->next;
 
   li->status = status;
-  li->op_err = op_err;
 
   /* Add LI to done list.  */
   li->next = ctx_done_list;
@@ -150,7 +147,7 @@ ctx_done (gpgme_ctx_t ctx, gpgme_error_t status, gpgme_error_t op_err)
    If a matching context could be found, return it.  Return NULL if no
    context could be found.  */
 static gpgme_ctx_t
-ctx_wait (gpgme_ctx_t ctx, gpgme_error_t *status, gpgme_error_t *op_err)
+ctx_wait (gpgme_ctx_t ctx, gpgme_error_t *status)
 {
   struct ctx_list_item *li;
 
@@ -167,8 +164,6 @@ ctx_wait (gpgme_ctx_t ctx, gpgme_error_t *status, gpgme_error_t *op_err)
       ctx = li->ctx;
       if (status)
 	*status = li->status;
-      if (op_err)
-	*op_err = li->op_err;
 
       /* Remove LI from done list.  */
       if (li->next)
@@ -206,18 +201,17 @@ _gpgme_wait_global_event_cb (void *data, gpgme_event_io_t type,
 	gpgme_error_t err = ctx_active (ctx);
 
 	if (err)
-	  /* An error occurred.  Close all fds in this context, and
+	  /* An error occured.  Close all fds in this context, and
 	     send the error in a done event.  */
-	  _gpgme_cancel_with_err (ctx, err, 0);
+	  _gpgme_cancel_with_err (ctx, err);
       }
       break;
 
     case GPGME_EVENT_DONE:
       {
-	gpgme_io_event_done_data_t done_data =
-	  (gpgme_io_event_done_data_t) type_data;
-
-	ctx_done (ctx, done_data->err, done_data->op_err);
+	gpgme_error_t *errp = (gpgme_error_t *) type_data;
+	assert (errp);
+	ctx_done (ctx, *errp);
       }
       break;
 
@@ -252,8 +246,7 @@ _gpgme_wait_global_event_cb (void *data, gpgme_event_io_t type,
    error occurs, NULL is returned and *STATUS is set to the error
    value.  */
 gpgme_ctx_t
-gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
-		gpgme_error_t *op_err, int hang)
+gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
 {
   do
     {
@@ -269,12 +262,10 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
       fdt.fds = malloc (i * sizeof (struct io_select_fd_s));
       if (!fdt.fds)
 	{
-          int saved_err = gpg_error_from_syserror ();
+	  int saved_errno = errno;
 	  UNLOCK (ctx_list_lock);
 	  if (status)
-	    *status = saved_err;
-	  if (op_err)
-	    *op_err = 0;
+	    *status = gpg_error_from_errno (saved_errno);
 	  return NULL;
 	}
       fdt.size = i;
@@ -290,12 +281,10 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
       nr = _gpgme_io_select (fdt.fds, fdt.size, 0);
       if (nr < 0)
 	{
-          int saved_err = gpg_error_from_syserror ();
+	  int saved_errno = errno;
 	  free (fdt.fds);
 	  if (status)
-	    *status = saved_err;
-	  if (op_err)
-	    *op_err = 0;
+	    *status = gpg_error_from_errno (saved_errno);
 	  return NULL;
 	}
 
@@ -305,12 +294,11 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	    {
 	      gpgme_ctx_t ictx;
 	      gpgme_error_t err = 0;
-	      gpgme_error_t local_op_err = 0;
 	      struct wait_item_s *item;
-
+	      
 	      assert (nr);
 	      nr--;
-
+	      
 	      item = (struct wait_item_s *) fdt.fds[i].opaque;
 	      assert (item);
 	      ictx = item->ctx;
@@ -322,12 +310,12 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	      UNLOCK (ctx->lock);
 
 	      if (!err)
-		err = _gpgme_run_io_cb (&fdt.fds[i], 0, &local_op_err);
-	      if (err || local_op_err)
+		err = _gpgme_run_io_cb (&fdt.fds[i], 0);
+	      if (err)
 		{
-		  /* An error occurred.  Close all fds in this context,
+		  /* An error occured.  Close all fds in this context,
 		     and signal it.  */
-		  _gpgme_cancel_with_err (ictx, err, local_op_err);
+		  _gpgme_cancel_with_err (ictx, err);
 
 		  /* Break out of the loop, and retry the select()
 		     from scratch, because now all fds should be
@@ -350,9 +338,7 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	      break;
 	  if (i == actx->fdt.size)
 	    {
-	      struct gpgme_io_event_done_data data;
-	      data.err = 0;
-	      data.op_err = 0;
+	      gpgme_error_t err = 0;
 
 	      /* FIXME: This does not perform too well.  We have to
 		 release the lock because the I/O event handler
@@ -363,7 +349,7 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 		 contexts to be released and call the DONE events
 		 afterwards.  */
 	      UNLOCK (ctx_list_lock);
-	      _gpgme_engine_io_event (actx->engine, GPGME_EVENT_DONE, &data);
+	      _gpgme_engine_io_event (actx->engine, GPGME_EVENT_DONE, &err);
 	      LOCK (ctx_list_lock);
 	      goto retry;
 	    }
@@ -371,7 +357,7 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
       UNLOCK (ctx_list_lock);
 
       {
-	gpgme_ctx_t dctx = ctx_wait (ctx, status, op_err);
+	gpgme_ctx_t dctx = ctx_wait (ctx, status);
 
 	if (dctx)
 	  {
@@ -383,8 +369,6 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	    ctx = NULL;
 	    if (status)
 	      *status = 0;
-	    if (op_err)
-	      *op_err = 0;
 	  }
       }
     }
@@ -392,10 +376,3 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 
   return ctx;
 }
-
-
-gpgme_ctx_t
-gpgme_wait (gpgme_ctx_t ctx, gpgme_error_t *status, int hang)
-{
-  return gpgme_wait_ext (ctx, status, NULL, hang);
-}
